#include "project.h"
#if !gpio__DISABLED
#include "stdio.h"
#include "tty_utils.h"
#include "gpio_utils.h"

static uint16_t gpio_loop = 1 ;
static uint16_t gpio_interval = 500 ; /* 500 ms */
static int      gpio_keep_on = 1 ;

f_list_type gpio_func_list[] = {
    { "help",     do_gpio_help,     "show help message" },
    { "loop",     do_gpio_loop,     "set loop count"    },    
    { "interval", do_gpio_interval, "set interval (ms)" },
    { "read",     do_gpio_read,     "read gpio input value and output value" },
    { "write",    do_gpio_write,    "write gpio output value" },
    { "status",   do_gpio_status,   "print gpio status" },
    { "repeat",   do_gpio_repeat,   "toggle gpio output and read gpio input" },
    { "return",   do_gpio_return,   "return to the previous mode" },
    { 0,         0,             0} /* list terminator */
} ;


void do_gpio(void) 
{
    func_ptr func ;
    gpio_keep_on = 1 ;
    
    do_gpio_help() ;
    
    prompt("gpio> ") ;
    
    while(gpio_keep_on) {
        if (get_line()) {
            func = get_func(gpio_func_list, str) ;
            if (func) {
                func() ;
            } else {
                print("Unknown Command: ") ;
                print(str) ;
                print("\n\r") ;
                do_gpio_help() ;
            }
            if (gpio_keep_on) {
                prompt("gpio> ") ;
            }
        }
    }
}

void do_gpio_help(void) 
{
    show_help(gpio_func_list) ;    
}

void do_gpio_loop(void) 
{
    sscanf(str, "%*s %hd", &gpio_loop) ;
}

void do_gpio_interval(void) 
{
    sscanf(str, "%*s %hd", &gpio_interval) ;    
}

void do_gpio_read(void)
{
    int in_value, out_value ;
    in_value = gpio_in_Read() ;
    out_value = gpio_out_Read() ;
    snprintf(str, STR_BUF_LEN, "IN: %d  OUT: %d\n", in_value, out_value) ;
    print(str) ;
}

void do_gpio_write(void) 
{
    int value ;
    sscanf(str, "%*s %d", &value) ;
    gpio_out_Write(value) ;
}

void do_gpio_repeat(void) 
{
    int i ;
    int out_value ;
    for (i = 0 ; i < gpio_loop ; i++ ) {
        out_value = gpio_out_Read() ;
        do_gpio_read() ;
        if (out_value) {
            out_value = 0 ;
        } else {
            out_value = 1 ;
        }
        gpio_out_Write(out_value) ;
        CyDelay(gpio_interval) ;
    }     
}

void do_gpio_status(void) 
{
    print("GPIO STATUS:\n\r") ;
    do_gpio_read() ;
    snprintf(str, STR_BUF_LEN, "loop: %d, interval: %d ms\n\r",
        gpio_loop, gpio_interval) ;
    print(str) ;
}
void do_gpio_return(void) 
{
    gpio_keep_on = 0 ;
}
#endif /* !gpio__DISABLED */
